from zzgui.qt5.widgets import (
    zzlabel,
    zzline,
    zzframe,
    zzcheck,
    zztoolbar,
    zztab,
    zztext,
    zzcode,
    zzbutton,
    zzrelation,
    zzradio,
    zzlookup,
    zzcombo,
    zztoolbutton,
    zzprogressbar,
    zzgrid,
    zzdate,
    zzspace,
    zzlist,
    zzspin,
)


(
    zzlabel,
    zzline,
    zzframe,
    zzcheck,
    zztoolbar,
    zztab,
    zztext,
    zzcode,
    zzbutton,
    zzrelation,
    zzradio,
    zzlookup,
    zzcombo,
    zztoolbutton,
    zzprogressbar,
    zzgrid,
    zzdate,
    zzspace,
    zzlist,
    zzspin,
)
